from django.views.generic.base import RedirectView
from .base import BaseViewMixin, BaseView, DisplayAttrMixin
from .forms import FormView, MatrixView, ObjectView
from .lists import ListView, CollectionView
